#include <ndn-cxx/data.hpp>
#include <ndn-cxx/face.hpp>
#include <ndn-cxx/security/key-chain.hpp>

#include <unistd.h>

ndn::Face face;

void
onInterest(const ndn::InterestFilter& filter, const ndn::Interest& interest) {
  usleep(1000000);
  ndn::KeyChain keyChain;

  std::string strData("NoData");

  ndn::Name dataName("/example/testApp");

  ndn::Block block(ndn::encoding::makeBinaryBlock(ndn::tlv::Content, reinterpret_cast<const uint8_t*>(strData.c_str()), strData.size()));

  ndn::EncodingBuffer buffer;
  buffer.prependBlock(std::move(block));

  const uint8_t* rawBuffer = buffer.buf();
  const uint8_t* segmentBegin = rawBuffer;
  const uint8_t* end = rawBuffer + buffer.size();

  uint64_t segmentNo = 0;
  do {
    const uint8_t* segmentEnd = segmentBegin + (ndn::MAX_NDN_PACKET_SIZE >> 1);
    if (segmentEnd > end) {
      segmentEnd = end;
    }

    ndn::Name segmentName(dataName);
    segmentName.appendSegment(segmentNo);

    std::shared_ptr<ndn::Data> data = std::make_shared<ndn::Data>(segmentName);
    data->setContent(segmentBegin, segmentEnd - segmentBegin);
    data->setFreshnessPeriod(ndn::time::seconds(1));

    segmentBegin = segmentEnd;
    if (segmentBegin >= end) {
      data->setFinalBlock(segmentName[-1]);
    }

    keyChain.sign(*data);
    face.put(*data);


    ++segmentNo;
  } while (segmentBegin < end);

}

int main() {
  face.setInterestFilter("/example/testApp",
                         bind(&onInterest, _1, _2),
                         nullptr, nullptr);

  face.processEvents();
  return 0;
}
